# Stock Prices Module Implementation Plan

Command to reread guidelines: Read /home/philkunz/.claude/CLAUDE.md

## Overview
Implementation of a stocks module for fetching current stock prices using various APIs. The architecture will support multiple providers, but we'll start with implementing only Yahoo Finance API. The design will make it easy to add additional providers (Alpha Vantage, IEX Cloud, etc.) in the future without changing the core architecture.

## Phase 1: Yahoo Finance Implementation

### 1.1 Research & Documentation
- [ ] Research Yahoo Finance API endpoints (no official API, using public endpoints)
- [ ] Document available data fields and formats
- [ ] Identify rate limits and restrictions
- [ ] Test endpoints manually with curl

### 1.2 Module Structure
```
ts/
├── index.ts                    # Main exports
├── plugins.ts                  # External dependencies
└── stocks/
    ├── index.ts                # Stocks module exports
    ├── classes.stockservice.ts # Main StockPriceService class
    ├── interfaces/
    │   ├── stockprice.ts      # IStockPrice interface
    │   └── provider.ts        # IStockProvider interface (for all providers)
    └── providers/
        ├── provider.yahoo.ts   # Yahoo Finance implementation
        └── (future: provider.alphavantage.ts, provider.iex.ts, etc.)
```

### 1.3 Core Interfaces
```typescript
// IStockPrice - Standardized stock price data
interface IStockPrice {
  ticker: string;
  price: number;
  currency: string;
  change: number;
  changePercent: number;
  timestamp: Date;
  provider: string;
}

// IStockProvider - Provider implementation contract
interface IStockProvider {
  name: string;
  fetchPrice(ticker: string): Promise<IStockPrice>;
  fetchPrices(tickers: string[]): Promise<IStockPrice[]>;
  isAvailable(): Promise<boolean>;
}
```

### 1.4 Yahoo Finance Provider Implementation
- [ ] Create YahooFinanceProvider class
- [ ] Implement HTTP requests to Yahoo Finance endpoints
- [ ] Parse response data into IStockPrice format
- [ ] Handle errors and edge cases
- [ ] Add request throttling/rate limiting

### 1.5 Main Service Class
- [ ] Create StockPriceService class with provider registry
- [ ] Implement provider interface for pluggable providers
- [ ] Register Yahoo provider (with ability to add more later)
- [ ] Add method for single ticker lookup
- [ ] Add method for batch ticker lookup
- [ ] Implement error handling with graceful degradation
- [ ] Design fallback mechanism (ready for multiple providers)

## Phase 2: Core Features

### 2.1 Service Architecture  
- [ ] Create provider registry pattern for managing multiple providers
- [ ] Implement provider priority and selection logic
- [ ] Design provider health check interface
- [ ] Create provider configuration system
- [ ] Implement provider discovery mechanism
- [ ] Add provider capability querying (which tickers/markets supported)

## Phase 3: Advanced Features

### 3.1 Caching System
- [ ] Design cache interface
- [ ] Implement in-memory cache with TTL
- [ ] Add cache invalidation logic
- [ ] Make cache configurable per ticker

### 3.2 Configuration
- [ ] Provider configuration (timeout, retry settings)
- [ ] Cache configuration (TTL, max entries)
- [ ] Request timeout configuration
- [ ] Error handling configuration

### 3.3 Error Handling
- [ ] Define custom error types
- [ ] Implement retry logic with exponential backoff
- [ ] Add circuit breaker pattern for failing providers
- [ ] Comprehensive error logging

## Phase 4: Testing

### 4.1 Unit Tests
- [ ] Test each provider independently
- [ ] Mock HTTP requests for predictable testing
- [ ] Test error scenarios
- [ ] Test data transformation logic

### 4.2 Integration Tests
- [ ] Test with real API calls (rate limit aware)
- [ ] Test provider fallback scenarios
- [ ] Test batch operations
- [ ] Test cache behavior

### 4.3 Performance Tests
- [ ] Measure response times
- [ ] Test concurrent request handling
- [ ] Validate cache effectiveness

## Implementation Order

1. **Week 1: Yahoo Finance Provider**
   - Research and test Yahoo endpoints
   - Implement basic provider and service
   - Create core interfaces
   - Basic error handling

2. **Week 2: Service Architecture**
   - Create extensible provider system
   - Implement provider interface
   - Add provider registration

3. **Week 3: Advanced Features**
   - Implement caching system
   - Add configuration management
   - Enhance error handling

4. **Week 4: Testing & Documentation**
   - Write comprehensive tests
   - Create usage documentation
   - Performance optimization

## Dependencies

### Required
- `@push.rocks/smartrequest` - HTTP requests
- `@push.rocks/smartpromise` - Promise utilities
- `@push.rocks/smartlog` - Logging

### Development
- `@git.zone/tstest` - Testing framework
- `@git.zone/tsrun` - TypeScript execution

## API Endpoints Research

### Yahoo Finance
- Base URL: `https://query1.finance.yahoo.com/v8/finance/chart/{ticker}`
- No authentication required
- Returns JSON with price data
- Rate limits unknown (need to test)
- Alternative endpoints to explore:
  - `/v7/finance/quote` - Simplified quote data
  - `/v10/finance/quoteSummary` - Detailed company data

## Success Criteria

1. Can fetch current stock prices for any valid ticker
2. Extensible architecture for future providers
3. Response time < 1 second for cached data
4. Response time < 3 seconds for fresh data
5. Proper error handling and recovery
6. Comprehensive test coverage (>80%)

## Notes

- Yahoo Finance provides free stock data without authentication
- **Architecture designed for multiple providers**: While only implementing Yahoo Finance initially, all interfaces, classes, and patterns are designed to support multiple stock data providers
- The provider registry pattern allows adding new providers without modifying existing code
- Each provider implements the same IStockProvider interface for consistency
- Future providers can be added by simply creating a new provider class and registering it
- Implement proper TypeScript types for all data structures
- Follow the project's coding standards (prefix interfaces with 'I')
- Use plugins.ts for all external dependencies
- Keep filenames lowercase
- Write tests using @git.zone/tstest with smartexpect syntax
- Focus on clean, extensible architecture for future growth

## Future Provider Addition Example

When ready to add a new provider (e.g., Alpha Vantage), the process will be:
1. Create `ts/stocks/providers/provider.alphavantage.ts`
2. Implement the `IStockProvider` interface
3. Register the provider in the StockPriceService
4. No changes needed to existing code or interfaces